@@ -1,3 +1,21 @@
+//! Lolicon API wrapper.
+//!
+//! Example usage:
+//!
+//! ```rust
+//! use lolicon_api::Request;
+//! use lolicon_api::R18;
+//! let req = Request::default()
+//!     .r18(R18::R18) // R-18
+//!     .num(1).unwrap() // 一张
+//!     .uid(vec![16731]).unwrap(); // 玉之けだま老师
+//! let req_url = String::from(req);
+//!
+//! assert_eq!(&req_url, "https://api.lolicon.app/setu/v2?&r18=1&num=1&uid=16731");
+//! ```
+//!
+//! **Note:** `req_url`'s fields does not graduated to be the same sort as building.
+
 pub use thiserror::Error;
 
 use convert::Argument;
@@ -25,11 +43,11 @@ pub struct Request {
     r18: Option<R18>,
     /// amount of result's artworks. 1-100 is legal.
     num: Option<u8>,
-    /// specified authors. at most 20s.
+    /// specified authors. at most 20s, at least one.
     uid: Option<Vec<u32>>,
-    /// Not very convenient. you should consider use tags instead.
+    /// Not very convenient. you should consider use `tag` instead.
     keyword: Keyword,
-    /// at most 20s
+    /// at most 20s, at least one.
     tag: Tag,
     /// available values were defined in its setter.
     size: Size,
@@ -43,44 +61,51 @@ pub struct Request {
     dsc: Option<bool>,
 }
 
-#[derive(Debug, Clone)]
-pub struct Keyword(Option<String>);
-
-#[derive(Debug, Clone)]
-pub struct Tag(Option<Vec<String>>);
-
-#[derive(Debug, Clone)]
-pub struct Size(Option<Vec<String>>);
-
-#[derive(Debug, Clone)]
-pub struct Proxy(Option<String>);
-
-#[derive(Debug, Clone)]
-pub struct DateAfter(Option<u64>);
-
-#[derive(Debug, Clone)]
-pub struct DateBefore(Option<u64>);
-
 #[derive(Copy, Clone, Debug)]
+/// Non-R18 by default.
 pub enum R18 {
     NonR18,
     R18,
     Mixin,
 }
 
+#[derive(Debug, Clone)]
+/// Not very convenient. you should consider use tags instead.
+pub struct Keyword(Option<String>);
+
+#[derive(Debug, Clone)]
+/// at most 20s, at least one.
+pub struct Tag(Option<Vec<String>>);
+
+#[derive(Debug, Clone)]
+/// available values were defined in its setter.
+pub struct Size(Option<Vec<String>>);
+
+#[derive(Debug, Clone)]
+/// proxy for `pixiv.net`, `i.pixiv.cat`, e.g. See [Lolicon](https://api.lolicon.app/#/setu?id=proxy) for detail.
+pub struct Proxy(Option<String>);
+
+#[derive(Debug, Clone)]
+/// Only show artworks after this UNIX time in millisecond.
+pub struct DateAfter(Option<u64>);
+
+#[derive(Debug, Clone)]
+/// Only show artworks before this UNIX time in millisecond.
+pub struct DateBefore(Option<u64>);
+
 impl std::default::Default for Request {
     fn default() -> Self {
         Request {
-            r18: None,
-            num: None,
+            r18: None, // Non-R18 by default
+            num: None, // 1 by default
             uid: None,
             keyword: Keyword(None),
             tag: Tag(None),
-            size: Size(None),
-            proxy: Proxy(None),
+            size: Size(None), // ["original"] by default
+            proxy: Proxy(None), // `i.pixiv.cat` by default
             date_after: DateAfter(None),
             date_before: DateBefore(None),
-            dsc: None,
+            dsc: None, // `false` by default
         }
     }
 }
@@ -92,7 +117,7 @@ impl Request {
         self
     }
 
-    /// set amount of result's artworks.
+    /// set amount of result's artworks. 1-100 is legal.
     pub fn num(mut self, amount: u8) -> Result<Self, LoliError> {
         match amount {
             1..=100 => {
@@ -148,10 +173,29 @@ impl Request {
         }
     }
 
+    /// proxy for `pixiv.net`, `i.pixiv.cat`, e.g. See [Lolicon](https://api.lolicon.app/#/setu?id=proxy) for detail.
     pub fn proxy(mut self, proxy: impl Into<String>) -> Self {
         self.proxy.0 = Some(proxy.into());
         self
     }
+
+    /// Only show artworks after this UNIX time in millisecond.
+    pub fn date_after(mut self, date_after: u64) -> Self {
+        self.date_after.0 = Some(date_after);
+        self
+    }
+
+    /// Only show artworks before this UNIX time in millisecond.
+    pub fn date_before(mut self, date_before: u64) -> Self {
+        self.date_before.0 = Some(date_before);
+        self
+    }
+
+    /// If this is `true`, some automatic convert between keywords and tags will be disabled.
+    pub fn dsc(mut self, dsc: bool) -> Self {
+        self.dsc = Some(dsc);
+        self
+    }
 }
 
 impl From<Request> for String {
@@ -174,10 +218,12 @@ impl From<Request> for String {
 }
 
 trait AddArgument {
+    /// to convert a argument into url field.
     fn add_argument(&mut self, object: impl Argument);
 }
 
 impl AddArgument for String {
+    /// to convert a argument into url field.
     fn add_argument(&mut self, object: impl Argument) {
         object.argument(self);
     }
